#
#  Copyright (c) 2020 Russell Smiley
#
#  This file is part of build_harness.
#
#  You should have received a copy of the MIT License along with build_harness.
#  If not, see <https://opensource.org/licenses/MIT>.
#

"""
BDD acceptance tests management.

Exports

*  ``bdd_acceptance_command`` function
"""

from .bdd_group import bdd_acceptance_command  # noqa: F401
